@@ -41,6 +41,7 @@
 #include <glib/gstdio.h>
 
 #include <gdk/gdk.h>
+#include <gdk/gdkx.h>
 
 #include <gtk/gtk.h>
 
@@ -66,7 +67,7 @@
 #include "brasero-drive.h"
 
 #include "brasero-misc.h"
-
+#include "brasero-pk.h"
 
 G_DEFINE_TYPE (BraseroBurnDialog, brasero_burn_dialog, GTK_TYPE_DIALOG);
 
@@ -103,6 +104,10 @@ struct BraseroBurnDialogPrivate {
 	gint64 total_size;
 	GSList *rates;
 
+	GMainLoop *loop;
+	gint wait_ready_state_id;
+	GCancellable *cancel_plugin;
+
 	gchar *initial_title;
 	gchar *initial_icon;
 
@@ -439,7 +444,7 @@ brasero_burn_dialog_wait_for_insertion (BraseroBurnDialog *dialog,
 
 	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
 
-	if (!GTK_WIDGET_VISIBLE (dialog)) {
+	if (!gtk_widget_get_visible (GTK_WIDGET (dialog))) {
 		gtk_widget_show (GTK_WIDGET (dialog));
 		hide = TRUE;
 	}
@@ -676,7 +681,7 @@ brasero_burn_dialog_image_error (BraseroBurn *burn,
 
 	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
 
-	if (!GTK_WIDGET_VISIBLE (dialog)) {
+	if (!gtk_widget_get_visible (GTK_WIDGET (dialog))) {
 		gtk_widget_show (GTK_WIDGET (dialog));
 		hide = TRUE;
 	}
@@ -819,7 +824,7 @@ brasero_burn_dialog_loss_warnings_cb (BraseroBurnDialog *dialog,
 
 	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
 
-	if (!GTK_WIDGET_VISIBLE (dialog)) {
+	if (!gtk_widget_get_visible (GTK_WIDGET (dialog))) {
 		gtk_widget_show (GTK_WIDGET (dialog));
 		hide = TRUE;
 	}
@@ -986,7 +991,7 @@ brasero_burn_dialog_eject_failure_cb (BraseroBurn *burn,
 
 	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
 
-	if (!GTK_WIDGET_VISIBLE (dialog)) {
+	if (!gtk_widget_get_visible (GTK_WIDGET (dialog))) {
 		gtk_widget_show (GTK_WIDGET (dialog));
 		hide = TRUE;
 	}
@@ -1306,8 +1311,7 @@ brasero_burn_dialog_dummy_success_cb (BraseroBurn *burn,
 	if (!GTK_WIDGET_MAPPED (dialog)) {
 		gtk_widget_show (GTK_WIDGET (dialog));
 		hide = TRUE;
-	}
-	else
+	} else
 		hide = FALSE;
 
 	g_timer_stop (priv->total_time);
@@ -1420,7 +1424,7 @@ brasero_burn_dialog_activity_stop (BraseroBurnDialog *dialog,
 
 	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
 
-	gdk_window_set_cursor (GTK_WIDGET (dialog)->window, NULL);
+	gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (dialog)), NULL);
 
 	markup = g_strdup_printf ("<b><big>%s</big></b>", message);
 	gtk_label_set_text (GTK_LABEL (priv->header), markup);
@@ -1456,6 +1460,66 @@ brasero_burn_dialog_activity_stop (BraseroBurnDialog *dialog,
 	gtk_window_set_urgency_hint (GTK_WINDOW (dialog), TRUE);
 }
 
+static BraseroBurnResult
+brasero_burn_dialog_install_missing_cb (BraseroBurn *burn,
+					BraseroPluginErrorType type,
+					const gchar *detail,
+					gpointer user_data)
+{
+	BraseroBurnDialogPrivate *priv = BRASERO_BURN_DIALOG_PRIVATE (user_data);
+	GCancellable *cancel;
+	BraseroPK *package;
+	GdkWindow *window;
+	gboolean res;
+	int xid = 0;
+
+	gtk_widget_set_sensitive (GTK_WIDGET (user_data), FALSE);
+
+	/* Get the xid */
+	window = gtk_widget_get_window (user_data);
+	xid = gdk_x11_drawable_get_xid (GDK_DRAWABLE (window));
+
+	package = brasero_pk_new ();
+	cancel = g_cancellable_new ();
+	priv->cancel_plugin = cancel;
+	switch (type) {
+		case BRASERO_PLUGIN_ERROR_MISSING_APP:
+			res = brasero_pk_install_missing_app (package, detail, xid, cancel);
+			break;
+
+		case BRASERO_PLUGIN_ERROR_MISSING_LIBRARY:
+			res = brasero_pk_install_missing_library (package, detail, xid, cancel);
+			break;
+
+		case BRASERO_PLUGIN_ERROR_MISSING_GSTREAMER_PLUGIN:
+			res = brasero_pk_install_gstreamer_plugin (package, detail, xid, cancel);
+			break;
+
+		default:
+			res = FALSE;
+			break;
+	}
+
+	if (package) {
+		g_object_unref (package);
+		package = NULL;
+	}
+
+	gtk_widget_set_sensitive (GTK_WIDGET (user_data), TRUE);
+	if (g_cancellable_is_cancelled (cancel)) {
+		g_object_unref (cancel);
+		return BRASERO_BURN_CANCEL;
+	}
+
+	priv->cancel_plugin = NULL;
+	g_object_unref (cancel);
+
+	if (!res)
+		return BRASERO_BURN_ERR;
+
+	return BRASERO_BURN_RETRY;
+}
+
 static BraseroBurnResult
 brasero_burn_dialog_setup_session (BraseroBurnDialog *dialog,
 				   GError **error)
@@ -1467,6 +1531,10 @@ brasero_burn_dialog_setup_session (BraseroBurnDialog *dialog,
 	brasero_burn_session_start (priv->session);
 
 	priv->burn = brasero_burn_new ();
+	g_signal_connect (priv->burn,
+			  "install-missing",
+			  G_CALLBACK (brasero_burn_dialog_install_missing_cb),
+			  dialog);
 	g_signal_connect (priv->burn,
 			  "insert-media",
 			  G_CALLBACK (brasero_burn_dialog_insert_disc_cb),
@@ -1619,7 +1687,7 @@ brasero_burn_dialog_notify_error (BraseroBurnDialog *dialog,
 	else
 		secondary = g_strdup (_("An unknown error occurred."));
 
-	if (!GTK_WIDGET_VISIBLE (dialog))
+	if (!gtk_widget_get_visible (GTK_WIDGET (dialog)))
 		gtk_widget_show (GTK_WIDGET (dialog));
 
 	message = brasero_burn_dialog_create_message (dialog,
@@ -1663,15 +1731,18 @@ brasero_burn_dialog_get_success_message (BraseroBurnDialog *dialog)
 	drive = brasero_burn_session_get_burner (priv->session);
 
 	if (brasero_track_type_get_has_stream (&priv->input)) {
-		if (BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (&priv->input))) {
-			if (priv->media & BRASERO_MEDIUM_DVD)
-				return g_strdup (_("Video DVD successfully burned"));
+		if (!brasero_drive_is_fake (drive)) {
+			if (BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (&priv->input))) {
+				if (priv->media & BRASERO_MEDIUM_DVD)
+					return g_strdup (_("Video DVD successfully burned"));
 
-			return g_strdup (_("(S)VCD successfully burned"));
+				return g_strdup (_("(S)VCD successfully burned"));
+			}
+			else
+				return g_strdup (_("Audio CD successfully burned"));
 		}
-		else
-			return g_strdup (_("Audio CD successfully burned"));
 
+		return g_strdup (_("Image successfully created"));
 	}
 	else if (brasero_track_type_get_has_medium (&priv->input)) {
 		if (!brasero_drive_is_fake (drive)) {
@@ -1760,7 +1831,7 @@ brasero_burn_dialog_notify_copy_finished (BraseroBurnDialog *dialog,
 	if (!GTK_WIDGET_VISIBLE (dialog))
 		gtk_widget_show (GTK_WIDGET (dialog));
 
-	main_message = g_strdup_printf (_("Copy #%i has been burned successfully."), ++ priv->num_copies);
+	main_message = g_strdup_printf (_("Copy #%i has been burned successfully."), priv->num_copies ++);
 	message = brasero_burn_dialog_create_message (dialog,
 	                                              GTK_MESSAGE_INFO,
 	                                              GTK_BUTTONS_CANCEL,
@@ -1805,7 +1876,12 @@ brasero_burn_dialog_success_run (BraseroBurnDialog *dialog)
 		GtkWidget *window;
 
 		window = brasero_session_edit_cover (priv->session, GTK_WIDGET (dialog));
-		gtk_dialog_run (GTK_DIALOG (window));
+		/* This strange hack is a way to workaround #568358.
+		 * At one point we'll need to hide the dialog which means it
+		 * will anwer with a GTK_RESPONSE_NONE */
+		while (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_NONE)
+			gtk_widget_show (GTK_WIDGET (dialog));
+
 		gtk_widget_destroy (window);
 		return FALSE;
 	}
@@ -1831,8 +1907,8 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 	 * - we wrote a merged session
 	 * - we were not already asked for a series of copy */
 	if (!priv->num_copies
-	&& !brasero_burn_session_is_dest_file (priv->session)
-	&&!(brasero_burn_session_get_flags (priv->session) & BRASERO_BURN_FLAG_MERGE)) {
+	&&  !brasero_burn_session_is_dest_file (priv->session)
+	&& !(brasero_burn_session_get_flags (priv->session) & BRASERO_BURN_FLAG_MERGE)) {
 		/* Useful button but it shouldn't be used for images */
 		make_another = gtk_dialog_add_button (GTK_DIALOG (dialog),
 						      _("Make _More Copies"),
@@ -1950,9 +2026,11 @@ brasero_burn_dialog_end_session (BraseroBurnDialog *dialog,
 	}
 	else if (priv->num_copies) {
 		retry = brasero_burn_dialog_notify_copy_finished (dialog, error);
+		if (!retry)
+			brasero_burn_dialog_notify_success (dialog);
 	}
 	else {
-		/* see if an image was created. If so, add it to GtkRecent */
+		/* see if an image was created. If so, add it to GtkRecent. */
 		if (brasero_burn_session_is_dest_file (priv->session)) {
 			GSList *tracks;
 
@@ -1966,7 +2044,7 @@ brasero_burn_dialog_end_session (BraseroBurnDialog *dialog,
 		}
 
 		retry = brasero_burn_dialog_notify_success (dialog);
-		priv->num_copies = retry;
+		priv->num_copies = retry * 2;
 	}
 
 	if (priv->burn) {
@@ -2113,17 +2191,18 @@ brasero_burn_dialog_record_session (BraseroBurnDialog *dialog)
 }
 
 static gboolean
-brasero_burn_dialog_wait_for_ready_state (BraseroBurnDialog *dialog)
+brasero_burn_dialog_wait_for_ready_state_cb (BraseroBurnDialog *dialog)
 {
 	BraseroBurnDialogPrivate *priv;
-	BraseroBurnResult result;
 	BraseroStatus *status;
 
 	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
 
 	status = brasero_status_new ();
-	result = brasero_burn_session_get_status (priv->session, status);
-	while (brasero_status_get_result (status) == BRASERO_BURN_NOT_READY) {
+	brasero_burn_session_get_status (priv->session, status);
+
+	if (brasero_status_get_result (status) == BRASERO_BURN_NOT_READY
+	||  brasero_status_get_result (status) == BRASERO_BURN_RUNNING) {
 		gdouble progress;
 		gchar *action;
 
@@ -2142,26 +2221,64 @@ brasero_burn_dialog_wait_for_ready_state (BraseroBurnDialog *dialog)
 							   progress,
 							   -1.0,
 							   priv->media);
+		g_object_unref (status);
 
+		/* Continue */
+		return TRUE;
+	}
+
+	if (priv->loop)
+		g_main_loop_quit (priv->loop);
+
+	priv->wait_ready_state_id = 0;
+
+	g_object_unref (status);
+	return FALSE;
+}
+
+static gboolean
+brasero_burn_dialog_wait_for_ready_state (BraseroBurnDialog *dialog)
+{
+	BraseroBurnDialogPrivate *priv;
+	BraseroBurnResult result;
+	BraseroStatus *status;
+
+	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
+
+	status = brasero_status_new ();
+	result = brasero_burn_session_get_status (priv->session, status);
+	if (result == BRASERO_BURN_NOT_READY || result == BRASERO_BURN_RUNNING) {
+		GMainLoop *loop;
+
+		loop = g_main_loop_new (NULL, FALSE);
+		priv->loop = loop;
+
+		priv->wait_ready_state_id = g_timeout_add_seconds (1,
+								   (GSourceFunc) brasero_burn_dialog_wait_for_ready_state_cb,
+								   dialog);
+		g_main_loop_run (loop);
+
+		priv->loop = NULL;
+
+		if (priv->wait_ready_state_id) {
+			g_source_remove (priv->wait_ready_state_id);
+			priv->wait_ready_state_id = 0;
+		}
+
+		g_main_loop_unref (loop);
+
+		/* Get the final status */
 		result = brasero_burn_session_get_status (priv->session, status);
 	}
-	brasero_status_free (status);
+
+	g_object_unref (status);
 
 	return (result == BRASERO_BURN_OK);
 }
 
-/**
- * brasero_burn_dialog_run:
- * @dialog: a #BraseroBurnDialog
- * @session: a #BraseroBurnSession
- *
- * Start burning the contents of @session.
- *
- * Return value: a #gboolean. TRUE if the operation was successfully carried out, FALSE otherwise.
- **/
-gboolean
-brasero_burn_dialog_run (BraseroBurnDialog *dialog,
-			 BraseroBurnSession *session)
+static gboolean
+brasero_burn_dialog_run_real (BraseroBurnDialog *dialog,
+			      BraseroBurnSession *session)
 {
 	BraseroBurnResult result;
 	BraseroBurnDialogPrivate *priv;
@@ -2175,6 +2292,9 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 	brasero_burn_session_get_input_type (session, &priv->input);
 	brasero_burn_dialog_update_media (dialog);
 
+	/* show it early */
+	gtk_widget_show (GTK_WIDGET (dialog));
+
 	/* wait for ready state */
 	if (!brasero_burn_dialog_wait_for_ready_state (dialog))
 		return FALSE;
@@ -2190,7 +2310,7 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 	priv->initial_icon = g_strdup (gtk_window_get_icon_name (GTK_WINDOW (dialog)));
 
 	do {
-		if (!GTK_WIDGET_VISIBLE (dialog))
+		if (!gtk_widget_get_visible (GTK_WIDGET (dialog)))
 			gtk_widget_show (GTK_WIDGET (dialog));
 
 		result = brasero_burn_dialog_record_session (dialog);
@@ -2219,6 +2339,55 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 	return (result == BRASERO_BURN_OK);
 }
 
+/**
+ * brasero_burn_dialog_run_multi:
+ * @dialog: a #BraseroBurnDialog
+ * @session: a #BraseroBurnSession
+ *
+ * Start burning the contents of @session. Once a disc is burnt, a dialog
+ * will be shown to the user and wait for a new insertion before starting to burn
+ * again.
+ *
+ * Return value: a #gboolean. TRUE if the operation was successfully carried out, FALSE otherwise.
+ **/
+gboolean
+brasero_burn_dialog_run_multi (BraseroBurnDialog *dialog,
+			       BraseroBurnSession *session)
+{
+	BraseroBurnResult result;
+	BraseroBurnDialogPrivate *priv;
+
+	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
+	priv->num_copies = 1;
+
+	result = brasero_burn_dialog_run_real (dialog, session);
+	return (result == BRASERO_BURN_OK);
+}
+
+
+/**
+ * brasero_burn_dialog_run:
+ * @dialog: a #BraseroBurnDialog
+ * @session: a #BraseroBurnSession
+ *
+ * Start burning the contents of @session.
+ *
+ * Return value: a #gboolean. TRUE if the operation was successfully carried out, FALSE otherwise.
+ **/
+gboolean
+brasero_burn_dialog_run (BraseroBurnDialog *dialog,
+			 BraseroBurnSession *session)
+{
+	BraseroBurnResult result;
+	BraseroBurnDialogPrivate *priv;
+
+	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
+	priv->num_copies = 0;
+
+	result = brasero_burn_dialog_run_real (dialog, session);
+	return (result == BRASERO_BURN_OK);
+}
+
 static gboolean
 brasero_burn_dialog_cancel_dialog (BraseroBurnDialog *toplevel)
 {
@@ -2273,6 +2442,11 @@ brasero_burn_dialog_cancel (BraseroBurnDialog *dialog,
 
 	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
 
+	if (priv->loop) {
+		g_main_loop_quit (priv->loop);
+		return TRUE;
+	}
+
 	if (!priv->burn)
 		return TRUE;
 
@@ -2428,6 +2602,16 @@ brasero_burn_dialog_finalize (GObject * object)
 
 	priv = BRASERO_BURN_DIALOG_PRIVATE (object);
 
+	if (priv->wait_ready_state_id) {
+		g_source_remove (priv->wait_ready_state_id);
+		priv->wait_ready_state_id = 0;
+	}
+
+	if (priv->cancel_plugin) {
+		g_cancellable_cancel (priv->cancel_plugin);
+		priv->cancel_plugin = NULL;
+	}
+
 	if (priv->initial_title) {
 		g_free (priv->initial_title);
 		priv->initial_title = NULL;
