@@ -498,7 +498,7 @@ brasero_burn_uri_thread (gpointer data)
 	grafts = g_slist_reverse (grafts);
 
 	/* remove all excluded starting by burn:// from the list */
-	for (src = brasero_track_data_get_excluded (BRASERO_TRACK_DATA (current), FALSE); src; src = src->next) {
+	for (src = brasero_track_data_get_excluded_list (BRASERO_TRACK_DATA (current)); src; src = src->next) {
 		gchar *uri;
 
 		uri = src->data;
@@ -734,8 +734,8 @@ brasero_burn_uri_init (BraseroBurnURI *obj)
 	priv->cond = g_cond_new ();
 }
 
-static BraseroBurnResult
-brasero_burn_uri_export_caps (BraseroPlugin *plugin, gchar **error)
+static void
+brasero_burn_uri_export_caps (BraseroPlugin *plugin)
 {
 	GSList *caps;
 
@@ -758,6 +758,4 @@ brasero_burn_uri_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (caps);
 
 	brasero_plugin_set_process_flags (plugin, BRASERO_PLUGIN_RUN_PREPROCESSING);
-
-	return BRASERO_BURN_OK;
 }
